/*****************************************************************************

	volume-src-limited
		
    Copyright (C) 2002  Lucas Walter

  	lucasw@icculus.org
	http://icculus.org/~lucasw

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

******************************************************************************/

#include <SDL.h>
#include <string>
#include <iostream>
#include <fstream>
#include <iterator>

#ifdef WIN32
#include <windows.h>
#endif

#include <GL/gl.h>												
#include <GL/glu.h>	
//#include <GL/glaux.h>

//#include <math.h>
#include <cmath>

#include "main.hpp"
#include "misc.hpp"
#include "movable.hpp"
#include "pointMass.hpp"
#include "objloader.hpp"

#ifdef WIN32
#define M_PI PI
#endif 

using namespace std;

objloader::objloader()
{	
/*
	counter = 0;

	numVertices = 0;
	numMtl		= 0;

	
	fprintf(stderr,"spaceship.obj init...");

	load("spaceship.obj");

	fprintf(stderr,"finished\n");*/
}


objloader::objloader(string objFile, float STEP,  coordSystem COORD_SYSTEM)
{	
	counter = 0;

	//numVertices = 0;
	//numNormals  = 0;
	//numMtl = 0;
	
	verbose = true;
	
	translucent = false;
	
	//drawBB = true;
	
	this->step = STEP;
	
	/// Strip directory info out of objFile
	string::size_type pos = objFile.find_last_of("/");
	this->subdir = objFile.substr(0,pos+1);
	objFile = objFile.substr(pos+1,objFile.size());

	if (verbose) cout << subdir + objFile << " init...";
	try { load(objFile); }
	catch (fileNotFound) { throw fileNotFound(); return;}
	if (verbose) cout << "finished\n";
	
	mass = 1;

	getBoundingBox();

	if (STEP != 0) {
		if (verbose) cout << "\tfinding moment of inertia..." << objFile << "\n";
		
		if 		(COORD_SYSTEM==CARTESIAN) 		findInteriorCubic();
		else if (COORD_SYSTEM==CYLINDRICAL) findInteriorCylindrical();
		else if (COORD_SYSTEM==SPHERICAL) 	findInteriorSpherical();
				
		if (verbose) cout << iStack.size() << " interior masses.\n";
		//findCenterOfMass();
		//centerPoints();
		//findIbody();
		//findSurface();
		
		if (verbose) cout << "...finished\n";
		//outputMassObj();
	}

	getBoundingBox();

	//makeList();

	if (verbose) cout << "\n";

}

objloader::~objloader()
{
	//for (int i = 0; i < mtls.size(); i++)
	//	delete mtls[i];
	
}

objloader& objloader::operator= (const objloader &obj1)
{

	return *this;
}


bool objloader::load(string objFile)
{
	/*
	// find the last forward slash in the string
	string::size_type lastSlashPosition;
	while (lastSlashPosition = objFile.find("/")) {}
	subdir = objFile.substr(0,lastSlashPosition);
	cout << "subdir = " << subdir << "\n";
	*/
	objFile = subdir + objFile;
	
	
	ifstream objStream(objFile.c_str());
	if (!objStream) { 
		cout << "file \"" << objFile << "\" not found\n";
		throw fileNotFound(); 
		return false; 
	}
	istream_iterator<string> objIt(objStream);
	istream_iterator<string> sentinel;

	vector<string> lines(objIt,sentinel);

	theMode = NONE;
	
	//	for_each(lines.begin(),lines.end(),&bvh::process);
	
	unsigned i;
	for (i =0; i< lines.size(); i++) {
		try { process(lines[i]); }
		catch (fileNotFound) { throw fileNotFound(); return false; }
	}
	
	if (verbose) cout << vertices.size() << " vertices\n";
	if (verbose) cout << normals.size() << " normals (should == vertices)\n";
	if (verbose) cout << mtls.size() << " materials\n";
	
	/// find face normals for collision detection and other uses
	vector3f normal;
	for (i = 0; i < mtls.size(); i++)
	{				
		for (unsigned j = 0; j < mtls[i]->faces.size(); j++)
		{
			int ind0 = mtls[i]->faces[j].vertIndices[0]-1;
			int ind1 = mtls[i]->faces[j].vertIndices[1]-1;
			int ind2 = mtls[i]->faces[j].vertIndices[2]-1;
				
			normal = Cross( vertices[ind1] - vertices[ind0],
				 		vertices[ind2] - vertices[ind0] );
			
			normal = normal/(normal.Length());

			mtls[i]->faces[j].normal = normal;

		}
	}
	
	return true;
}
	
void objloader::process(string line)
{	
	int index; 
	string::size_type slashes;
	
	//cout << "Mode = " << theMode << "\n";
	if ((line.size()) && (line.substr(0,1) != "#"))
	{
		if (line == "mtllib") {
			theMode = MTLLIB;
		} else if (line == "v") {
			theMode = VERTEX;
			tempVectorIndex = 0;
		} else if (line == "vn") {
			theMode = NORMAL; 
			tempVectorIndex = 0;
		} else if (line == "f") {
			theMode = FACE;
			tempVectorIndex = 0;
		} else switch (theMode) {
		
			case (MTLLIB):
				if (verbose) cout << "\n\tparsing .mtl file: " << subdir + line << "...";
				mtlFile = line;
				
				if (!matchMtl(mtlIndex, mtlFile)) {
					try { loadMtl(mtlFile); }
					catch (fileNotFound) { throw fileNotFound(); return; }
						
					matchMtl(mtlIndex, mtlFile);
				}
				if (verbose) cout << ".mtl " << mtlIndex << " done\n";
				theMode = NONE;
			break;
			
			case (VERTEX):
				
				tempVector.vertex[tempVectorIndex] = atof(line.c_str());
				//cout << tempVector.vertex[tempVectorIndex] << "\n";
				tempVectorIndex++;
				if (tempVectorIndex >=3) {
					theMode = NONE;
					vertices.push_back(tempVector);	
				}
				
			break;
			
			case (NORMAL):
				tempVector.vertex[tempVectorIndex] = atof(line.c_str());
				tempVectorIndex++;
				if (tempVectorIndex >=3) {
					theMode = NONE;
					normals.push_back(tempVector);
				}
			
			break;
			
			case (FACE):
				/// face indices are denoted index//index (redundantly) in the .obj format
				slashes = line.find("//");
				index = atoi(line.substr(0,slashes).c_str());
				//cout << slashes << " " << index << "\n";
				tempTriangle.vertIndices[tempVectorIndex] = index;
				tempVectorIndex++;
				if (tempVectorIndex >= 3) {
					theMode = NONE;
					mtls[mtlIndex]->faces.push_back(tempTriangle);	
				}
				
			break;
			
			case (NONE):
				break;
		}
		
	}
	
		
}

void objloader::processMtl(string line, material *mtl)
{	
	if ((line.size()) && (line.substr(0,1) != "#"))
	{
		if (line == "newmtl") {
			theMtlMode = NEWMTL;
		} else if (line == "Ns") {
			theMtlMode = NS;
		} else	if (line == "d") {
			theMtlMode = D;
		} else	if (line == "illum") {
			theMtlMode = ILLUM;
		} else	if (line == "Kd") {
			theMtlMode = KD;
			kIndex = 0;
		} else	if (line == "Ka") {
			theMtlMode = KA;
			kIndex = 0;
		} else	if (line == "Ks") {
			theMtlMode = KS;
			kIndex = 0;
		} else switch (theMtlMode) {
		
			case KD:
				mtl->Kd[kIndex] = atof(line.c_str());
				kIndex++;
				if (kIndex >= 3) {
					mtl->Kd[3] = 1.0f;
					/*cout << mtl->name << " " << mtl->Kd[0] << " " << 
						mtl->Kd[1] << " " << mtl->Kd[2] << " " <<
						mtl->Kd[3] << "\n";*/
					theMtlMode = NONEM;
				}
				break;
				
			case KA:
				mtl->Ka[kIndex] = atof(line.c_str());
				kIndex++;
				if (kIndex >= 3) {
					mtl->Ka[3] = 1.0f;
					theMtlMode = NONEM;
				}
				break;
				
			case KS:
				mtl->Ks[kIndex] = atof(line.c_str());
				kIndex++;
				if (kIndex >= 3) {
					mtl->Ks[3] = 1.0f;
					theMtlMode = NONEM;
				}
				break;
				
			case (NONEM):
			case (NEWMTL):
			case (NS):
			case (D):
			case (ILLUM):
				break;
				
		}
	}
}

void objloader::loadMtl(string mtlFile)
{	
	ifstream objStream((subdir + mtlFile).c_str());
	
	if (!objStream) { 
		cout << "file \"" << subdir + mtlFile << "\" not found\n";
		throw fileNotFound(); 
		return;
	}
	
	istream_iterator<string> objIt(objStream);
	istream_iterator<string> sentinel;

	vector<string> lines(objIt,sentinel);

	theMtlMode = NONEM;
	
	material* newMtl = new material;
	newMtl->name = mtlFile;
	
	newMtl->Ke[0] = 0;
	newMtl->Ke[1] = 0;
	newMtl->Ke[2] = 0;
	newMtl->Ke[3] = 1.0f;
	
	unsigned i;
	for (i = 0; i< lines.size(); i++) {
		processMtl(lines[i],newMtl);
	}
	
	mtls.push_back(newMtl);
}

bool objloader::matchMtl(unsigned &index, string name)
{
	unsigned m;
	
	for (m = 0; m < (mtls.size()); m++) {
		
		if (name == mtls[m]->name) {
			index = m;
			return true;
		}
	}
	//cout << (int)mtls.size() <<" mtlssize \n";
	
	return false;
}


void objloader::setMass(float newMass)
{
	if (step != 0)
	{
		Ibody = (Ibody*newMass)/mass;
		IbodyInv = Ibody.Inverse();
	}

	mass = newMass;

}

/*void objloader::InitTexture()
{
//	image = auxDIBImageLoad("spaceshipTex2.bmp");
//LoadBMP("spaceshipTex2.bmp");

//	GLfloat env_color[4];
//	GLfloat border_color[4];
    
	glGenTextures(1, &texture[0]);
	glBindTexture(GL_TEXTURE_2D, texture[0]);

	glTexImage2D(GL_TEXTURE_2D, 0, 3, image->sizeX, image->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, image->data);


    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
//	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, env_color);

//	glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, border_color);

    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, image->sizeX, image->sizeX, GL_RGB, GL_UNSIGNED_BYTE, image->data);

}
*/

void objloader::update(void)
{
	getAABB();

	/*
	if (counter <90)
		location.RotateX(1);
	else if (counter <180)
		location.RotateY(1);
	else if (counter <300)
		location.Translate(0,0,0.2f);
		//location.RotateY(1);
		//location.RotateZ(1);
	else if (counter < 290)
		location.RotateX(1);
	else if (counter < 400)
		location.Translate(0,0,0.2f);
	else
		counter = -1;

	counter++;*/


}

void objloader::draw(void)
{
	return;
}


void objloader::getBoundingBox()
{
	/*	
	for (int j = 0; j<3;j++)
	{
		// lower right bound
		boundingBox[0].vertex[j] = test[0]->location.vertex[j];

		// upper right bound
		boundingBox[6].vertex[j] = test[0]->location.vertex[j];
	}
	*/
	
	boundingBox[0] = vertices[1];
	boundingBox[6] = vertices[1];
			
	// find extremities
	for (unsigned i = 1; i < vertices.size(); i++)
	{
		for (int j = 0; j<3;j++)
		{
		// lower left   minimum bound 
		if (boundingBox[0].vertex[j] > vertices[i].vertex[j])
			boundingBox[0].vertex[j] = vertices[i].vertex[j];

		// upper right  maximum bound 
		if (boundingBox[6].vertex[j] < vertices[i].vertex[j])
			boundingBox[6].vertex[j] = vertices[i].vertex[j];
	
		}
	}

	centerBB = (boundingBox[0] + boundingBox[6])/2;
	edgesBB = boundingBox[6] - centerBB;

/*	Log("%f %f %f\n\n",
		edgesBB.vertex[0],
		edgesBB.vertex[1],
		edgesBB.vertex[2]);*/



	// having found maximums, populate rest of corners
	boundingBox[1].vertex[0] = boundingBox[0].vertex[0];
	boundingBox[1].vertex[1] = boundingBox[0].vertex[1];
	boundingBox[1].vertex[2] = boundingBox[6].vertex[2];
	
	boundingBox[2].vertex[0] = boundingBox[0].vertex[0];
	boundingBox[2].vertex[1] = boundingBox[6].vertex[1];
	boundingBox[2].vertex[2] = boundingBox[6].vertex[2];
	
	boundingBox[3].vertex[0] = boundingBox[0].vertex[0];
	boundingBox[3].vertex[1] = boundingBox[6].vertex[1];
	boundingBox[3].vertex[2] = boundingBox[0].vertex[2];
		
	boundingBox[4].vertex[0] = boundingBox[6].vertex[0];
	boundingBox[4].vertex[1] = boundingBox[0].vertex[1];
	boundingBox[4].vertex[2] = boundingBox[0].vertex[2];
	
	boundingBox[5].vertex[0] = boundingBox[6].vertex[0];
	boundingBox[5].vertex[1] = boundingBox[0].vertex[1];
	boundingBox[5].vertex[2] = boundingBox[6].vertex[2];
	

	boundingBox[7].vertex[0] = boundingBox[6].vertex[0];
	boundingBox[7].vertex[1] = boundingBox[6].vertex[1];
	boundingBox[7].vertex[2] = boundingBox[0].vertex[2];

}

//void objloader::solveEq(vector3f *one,vector3f *two, vector3f *three,


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void objloader::findInteriorCubic(void)
{
	vector3f testLineMin,testLineMax,normal,testPoint;
				
	float xf,yf,zf;

	for (xf = boundingBox[0].vertex[0]-step; xf < boundingBox[6].vertex[0]+step; xf+=step) {
	for (zf = boundingBox[0].vertex[2]-step; zf < boundingBox[6].vertex[2]+step; zf+=step) {
	for (yf = boundingBox[0].vertex[1]-step; yf < boundingBox[6].vertex[1]+step; yf+=step) {
		
		testPoint.vertex[0] = xf;
		testPoint.vertex[1] = yf;
		testPoint.vertex[2] = zf;
		
		testInteriorPoint(testPoint);
	}	
	}
	}
}

void objloader::findInteriorCylindrical(void)
{
	/// find out how far out to find testPoints.
	float maxRadius1 = sqrtf(pow(boundingBox[6].vertex[0],2) + pow(boundingBox[6].vertex[2],2));
	float maxRadius2 = sqrtf(pow(boundingBox[0].vertex[0],2) + pow(boundingBox[0].vertex[2],2));
	float maxRadius = (maxRadius1 > maxRadius2) ? maxRadius1 : maxRadius2;
	
	vector3f testPoint;
	
	for (float layer = 0; layer*step < maxRadius; layer++) {
	for (float yf = boundingBox[0].vertex[1]-step; yf < boundingBox[6].vertex[1]+step; yf+=step) {
		
	//for (float theta = 0; theta < 2*M_PI && layer != 0; theta +=2*M_PI/((float)(int)(layer*2*M_PI)) {	
	for (float theta = 0; theta < 2*M_PI && layer != 0; theta +=2*M_PI/(layer*6)) {
	
		/// layer*step is the current radial position 
		testPoint.vertex[0] = layer*step*cos(theta);
		testPoint.vertex[1] = yf;
		testPoint.vertex[2] = layer*step*sin(theta);
		
		testInteriorPoint(testPoint);
	}
	/// Don't test multiple points that are just going to be on top of each other
	if (layer == 0) {
		/// layer*step is the current radial position 
		testPoint.vertex[0] = 0;
		testPoint.vertex[1] = yf;
		testPoint.vertex[2] = 0;
		
		//unsigned oldSize = iStack.size();
		testInteriorPoint(testPoint);
		//cout << "new Point = " << iStack.size()-oldSize << "\n";
		
	}
	
	}
	}
	
}

void objloader::findInteriorSpherical(void)
{
	float maxRadius1 = boundingBox[6].Length();
	float maxRadius2 = boundingBox[0].Length();
	float maxRadius = (maxRadius1 > maxRadius2) ? maxRadius1 : maxRadius2;

	vector3f testPoint;
	float numIncrementTemp, phiIncrement;
	
	/// Test the origin
	testInteriorPoint(vector3f(0,0,0));
	
	for (float layer = 1; layer*step < maxRadius; layer++) {
	//for (float theta = 0; theta < M_PI + M_PI/(layer*6); theta +=2*M_PI/((float)(int)(layer*2*M_PI)) {		
	for (float theta = 0; theta < M_PI + M_PI/(layer*6); theta +=M_PI/(layer*3)) {
		
			numIncrementTemp = (float)((int)(2*M_PI*layer*sin(theta)));
			phiIncrement = 2*M_PI/numIncrementTemp;	
		
		/// treat theta ~= M_PI differently- don't give it multiple overlapping points	
		if ((theta > M_PI/(layer*6)) && (theta < M_PI - M_PI/(layer*6) )) {	 
			for (float phi = 0; phi < 2*M_PI; phi += phiIncrement ) { 
				
				/// Convert spherical coordinates to cartesian
				/// layer*step is the current radial position 
				testPoint.vertex[0] = layer*step*cos(phi)*sin(theta);
				testPoint.vertex[1] = layer*step*cos(theta);
				testPoint.vertex[2] = layer*step*sin(phi)*sin(theta);
				
				testInteriorPoint(testPoint);
			}	
		} else {
			/// test north and south poles
			testPoint.vertex[0] = 0;
			testPoint.vertex[1] = layer*step;
			testPoint.vertex[2] = 0;
			
			testInteriorPoint(testPoint);
			
			testPoint.vertex[0] = 0;
			testPoint.vertex[1] = -layer*step;
			testPoint.vertex[2] = 0;
			
			testInteriorPoint(testPoint);
		
		}	
	}
	}	
}

void objloader::testInteriorPoint(vector3f testPoint)
{
	vector3f testLineMin, testLineMax, normal;
	
	float xf = testPoint.vertex[0]; 
	float yf = testPoint.vertex[1];
	float zf = testPoint.vertex[2];
	
	/**	if xf,yf,zf is inside the object,
		that is a vertical (or any line really) intersecting the obj through
		that point has surface intersection vectors that both point away from
		the test point1
	*/
	//		iStack.push_back(new vector3f(xf,yf,zf));
		
	/// make testLineMax-testLineMin a vector that spans
	/// the bBox from bottom to top vertically
	testLineMin.vertex[0] = xf;
	testLineMin.vertex[1] = boundingBox[0].vertex[1]-step;
	testLineMin.vertex[2] = zf;

	testLineMax.vertex[0] = xf;
	testLineMax.vertex[1] =	boundingBox[6].vertex[1]+step;
	testLineMax.vertex[2] = zf;



	vector3f lowerIntersection,upperIntersection,
		lowerIntersectionNormal,upperIntersectionNormal;

	// Initialize lower and upperIntersection to endpoints of testLine;
	lowerIntersection = testLineMin;
	upperIntersection = testLineMax;

	// test newLocataion-location against all faces in obj
	// and find the two closest points and normals of
	// intersection
	for (unsigned i = 0; i < mtls.size(); i++) {				
		for (unsigned j = 0; j < mtls[i]->faces.size(); j++) {
			// subtract 1 because the .obj file starts
			// with 1 and not 0
			int ind0 = mtls[i]->faces[j].vertIndices[0]-1;
			int ind1 = mtls[i]->faces[j].vertIndices[1]-1;
			int ind2 = mtls[i]->faces[j].vertIndices[2]-1;

			float gamma = .001;
			// does the triangle even span the test point
			if (
			   !((vertices[ind0].vertex[2] - gamma >= zf) &&	 
				 (vertices[ind1].vertex[2] - gamma >= zf) &&
				 (vertices[ind2].vertex[2] - gamma >= zf)) 
				 
				 &&
			  
			   !((vertices[ind2].vertex[0] - gamma >= xf) &&
				 (vertices[ind1].vertex[0] - gamma >= xf) &&
				 (vertices[ind0].vertex[0] - gamma >= xf)) 
				 
				 &&

			   !((vertices[ind0].vertex[2] + gamma <= zf) &&	 
				 (vertices[ind1].vertex[2] + gamma <= zf) &&
				 (vertices[ind2].vertex[2] + gamma <= zf)) 
				 
				 &&
			  
			   !((vertices[ind2].vertex[0] + gamma <= xf) &&
				 (vertices[ind1].vertex[0] + gamma <= xf) &&
				 (vertices[ind0].vertex[0] + gamma <= xf)) 

			 ) {
				 
	
				normal = mtls[i]->faces[j].normal;
				
				
				/// see where the intersection is 
				/// (should be between 0 and 1 if intersection 
				/// occurs within the bounding box, but don't bother
				/// testing that)
				float mu = pointMass::planeTest(testLineMin,testLineMax,
												vertices[ind1],
												normal);
				
				/// mu == 0 means the face is perpendicular
				if (mu != 0) {
					

					vector3f intersection = testLineMin + (testLineMax-testLineMin)*mu;
					
					/// see if the intersection point is inside the triangle
					float sumRadians = pointMass::triangleTest(intersection,
								vertices[ind0],
								vertices[ind1],
								vertices[ind2] );
					
					//cout << sumRadians/M_PI << "\n";
					
					/// sumRadians should be around 2*PI, but we should give
					/// it a little slack
					float gamma2 = PI;
					if ((sumRadians >= 2*M_PI - gamma2) && (sumRadians <= 2*M_PI + gamma2)) {
						/// an intersection has occurred
						
						/// is it the closest above the testPoint?
						if ((intersection.vertex[1] > yf) &&
							(intersection.vertex[1] < upperIntersection.vertex[1])) {
							upperIntersection = intersection;
							upperIntersectionNormal = normal;
								//cout << "upper\n";
								
						}
						/// is it the closest below the testPoint?
						else if ((intersection.vertex[1] < yf) &&
							(intersection.vertex[1] > lowerIntersection.vertex[1])) {
							lowerIntersection = intersection;
							lowerIntersectionNormal = normal;
								//cout << "lower\n";								
							
						}			
					} /// sumRadians  inside triangle test
							
				} /// mu != 0

			} /// span test 

		} /// faces loop
	} /// mtls loop

		
	/// now with the two closest normals, test if both are pointing
	/// away and thus the testPoint is inside the object
	
	/*
	testPoint.print();
	(upperIntersection - testPoint).print();
	(lowerIntersection - testPoint).print();
	upperIntersectionNormal.print();
	lowerIntersectionNormal.print();
	*/
	
	/// if the normal is zero, that means that the sumRadians test was not passed above
	if ((dot((upperIntersection - testPoint),upperIntersectionNormal) > 0 ) &&
		(dot((lowerIntersection - testPoint),lowerIntersectionNormal) > 0 )  ) {				

		
		/// the testPoint is behind the two surfaces and therefore
		/// inside the object
			
		//iStack.push_back(vector3f(xf,yf,zf));
			
/*			cout << iStack[iStack.size() - 1].vertex[0] << " " <<
				iStack[iStack.size() - 1].vertex[1] << " " <<
				iStack[iStack.size() - 1].vertex[2] << "\n";
*/
		/// these are just for debugging
		upperI.push_back(upperIntersection);
		lowerI.push_back(lowerIntersection);
			
		iStack.push_back(testPoint);
	
	}
}


/// loop through the mass list and find the average in each dimension

/** foreach( vector3f in iStack ), add to sum and then average for center of mass
 it'll be different than the center of average of all the surface vertices because
 it's for a volume and not a surface
 with center of mass, cycle through iStack again and sum up square of distance to center
 and then multiply by the mass of each point in the iStack -> objMass/iStack.size()
 the mass of the obj will have to be arbitrarily chosen - a few thousand kg?
*/
void objloader::findCenterOfMass(void)
{
	vector3f sum;


	std::vector<vector3f>::iterator it = iStack.begin();
	for (unsigned int i =0; i < iStack.size(); i++ )
	{
		sum = sum + (*it);
		std::advance(it, 1); 
	}
		
	centerOfMass = sum/static_cast<float>(iStack.size());
}


/// move the object so it rotates about it's centerOfMass
void objloader::centerPoints(void)
{
	unsigned i;
	for (i = 0; i < vertices.size(); i++)
	{
		vertices[i] = vertices[i] - centerOfMass;

	}

	std::vector<vector3f>::iterator it = iStack.begin();
	for (i =0; i < iStack.size(); i++ )
	{
		(*it) = (*it) - centerOfMass;
		std::advance(it, 1); 
	}
		
	/// this should be 0,0,0 now
	findCenterOfMass();


}


void objloader::findIbody(void)
{
	float	Ixx,Iyy,Izz,	/// moment of inertia
			Ixy,Ixz,Iyz;	/// products of intertia

	Ixx = Iyy = Izz = Ixy = Ixz = Iyz = 0;

	std::vector<vector3f>::iterator it = iStack.begin();
	for (unsigned int i =0; i < iStack.size(); i++ )
	{
		Ixx = Ixx + (it->vertex[1]*it->vertex[1])+(it->vertex[2]*it->vertex[2]);	/// y^2 + z^2
		Iyy = Iyy + (it->vertex[0]*it->vertex[0])+(it->vertex[2]*it->vertex[2]);	/// x^2 + z^2
		Izz = Izz + (it->vertex[0]*it->vertex[0])+(it->vertex[1]*it->vertex[1]);	/// x^2 + y^2

		Ixy = Ixy - (it->vertex[0] * it->vertex[1]);	///x*y
		Ixz = Ixz - (it->vertex[0] * it->vertex[2]);	///x*z
		Iyz = Iyz - (it->vertex[1] * it->vertex[2]);	///y*z

		std::advance(it, 1); 
	}


	///  | Ixx Ixy Ixz |
	///  | Iyx Iyy Iyz |
	///  | Izx Izy Izz |

	Ibody.matrix[0] = Ixx;	Ibody.matrix[3] = Ixy;	Ibody.matrix[6] = Ixz;
	Ibody.matrix[1] = Ixy;	Ibody.matrix[4] = Iyy;	Ibody.matrix[7] = Iyz;
	Ibody.matrix[2] = Ixz;	Ibody.matrix[5] = Iyz;	Ibody.matrix[8] = Izz;


	Ibody = (Ibody*mass)/static_cast<const float>(iStack.size());
	IbodyInv = Ibody.Inverse();
	
	/// It turns out for rectangular bodies and similar,
	/// the moments dominate and the products are 1/100 as big
	/// the sphereical bodies it's more like 1/10
}



void objloader::findSurface(void)
{
	bool neighbor[6];

	int k;
	for (unsigned int i =0; i < iStack.size(); i++ )
	{
		for (k = 0; k<4; k++)
			neighbor[k] = false;

		for (k = 4; k<6; k++)
			neighbor[k] = true;

		for (unsigned int j =0; j < iStack.size(); j++ )
		{
			if (i!=j)
			{
					 if	(iStack[j] ==  (iStack[i] + vector3f(0,0,step)) )
					neighbor[0] = true;

				else if (iStack[j] ==  (iStack[i] + vector3f(0,0,-step)) )
					neighbor[1] = true;

				else if (iStack[j] ==  (iStack[i] + vector3f(0,step,0)) ) 
					neighbor[2] = true;

				else if (iStack[j] ==  (iStack[i] + vector3f(0,-step,0)) )
					neighbor[3] = true;

				else if (iStack[j] ==  (iStack[i] + vector3f(step,0,0)) )
					neighbor[4] = true;

				else if (iStack[j] ==  (iStack[i] + vector3f(-step,0,0)) )
					neighbor[5] = true;
/*			
				if (iStack[j] ==  (iStack[i] + vector3f(-step,-step,-step)) ) 
					neighbor[0] = true;

				else if (iStack[j] ==  (iStack[i] + vector3f(-step,-step,0)) )
					neighbor[1] = true;

				else if (iStack[j] ==  (iStack[i] + vector3f(-step,-step,step)) )
					neighbor[2] = true;


				else if (iStack[j] ==  (iStack[i] + vector3f(-step,0,-step)) )
					neighbor[3] = true;

				else if (iStack[j] == (iStack[i] + vector3f(-step,0,0)) )
					neighbor[4] = true;

				else if (iStack[j] == (iStack[i] + vector3f(-step,0,step)) )
					neighbor[5] = true;

				
				else if (iStack[j] == (iStack[i] + vector3f(-step,step,-step)) )
					neighbor[6] = true;

				else if (iStack[j] == (iStack[i] + vector3f(-step,step,0)) )
					neighbor[7] = true;

				else if (iStack[j] == (iStack[i] + vector3f(-step,step,step)) )
					neighbor[8] = true;

				//

				else if (iStack[j] == (iStack[i] + vector3f(0,-step,-step)) )
				{
				//	vector3f some = iStack[j];
				//	some = iStack[i];
					//some = iStack[i] + vector3f(0,-step,-step);
					neighbor[9] = true;
				}

				else if (iStack[j] == (iStack[i] + vector3f(0,-step,0)) )
					neighbor[10] = true;

				else if (iStack[j] == (iStack[i] + vector3f(0,-step,step)) )
					neighbor[11] = true;


				else if (iStack[j] == (iStack[i] + vector3f(0,0,-step)) )
					neighbor[12] = true;

				//else if (iStack[i] == (iStack[i] + vector3f(-0,0,0))
				//	neighbor[4] = true;

				else if (iStack[j] == (iStack[i] + vector3f(0,0,step)) )
					neighbor[13] = true;

				
				else if (iStack[j] == (iStack[i] + vector3f(0,step,-step)) )
					neighbor[14] = true;

				else if (iStack[j] == (iStack[i] + vector3f(0,step,0)) )
					neighbor[15] = true;

				else if (iStack[j] == (iStack[i] + vector3f(0,step,step)) )
					neighbor[16] = true;

				//

				else if (iStack[j] == (iStack[i] + vector3f(-step,-step,-step)) )
					neighbor[17] = true;

				else if (iStack[j] == (iStack[i] + vector3f(step,-step,0)) )
					neighbor[18] = true;

				else if (iStack[j] == (iStack[i] + vector3f(step,-step,step)) )
					neighbor[19] = true;


				else if (iStack[j] == (iStack[i] + vector3f(step,0,-step)) )
					neighbor[20] = true;

				else if (iStack[j] == (iStack[i] + vector3f(step,0,0)) )
					neighbor[21] = true;

				else if (iStack[j] == (iStack[i] + vector3f(step,0,step)) )
					neighbor[22] = true;

				
				else if (iStack[j] == (iStack[i] + vector3f(step,step,-step)) )
					neighbor[23] = true;

				else if (iStack[j] == (iStack[i] + vector3f(step,step,0)) )
					neighbor[24] = true;

				else if (iStack[j] == (iStack[i] + vector3f(step,step,step)) )
					neighbor[25] = true;
*/
			}
		}

		bool isInterior = true;
		for (int m = 0; m<6;m++)
		{
			if (!neighbor[m])
				isInterior = false;
		}
		
		if (!isInterior)
			surface.push_back(iStack[i]);
		else
			isInterior = true;
	}
}

	


/// unique file type same as obj but stores 'vm' mass points
void objloader::outputMassObj(void)
{
//	string objname;
//	objname += name;
	ofstream fileWrite( "some.mbj" ,ios::out);
	std::vector<vector3f>::iterator it = iStack.begin();
	for (unsigned int i =0; i < iStack.size(); i++ )
	{
		fileWrite << "vm " 
			<< (*it).vertex[0] << " " 
			<< (*it).vertex[1] << " "
			<< (*it).vertex[2] << " "
									  << "\n";
		std::advance(it, 1); 
	}
 
	fileWrite.close();

}

